@@ -11,13 +11,11 @@ import json
 import locale
 import os
 import time
-import sys
-import xml.etree.ElementTree as ET
 import zipfile
+import requests
 
 from transcriptic.english import AutoprotocolParser
 from transcriptic.config import Connection
-from transcriptic.objects import ProtocolPreview
 from transcriptic.util import iter_json, flatmap, ascii_encode
 from transcriptic import routes
 from os.path import isfile
@@ -138,15 +136,31 @@ def cli(ctx, api_root, email, token, organization, config):
     note that the order of preference is: --flag, environment then config file.
     
     Example: `transcriptic --organization "my_org" projects` >> 
-    `export USER_ORGANIZATION="my_org"` >> `"organization_id": "my_org" in ~/.transcriptic
+    `export USER_ORGANIZATION="my_org"` >> `"organization_id": "my_org" in
+     ~/.transcriptic
     """
-    if ctx.invoked_subcommand in ['login', 'compile', 'preview', 'summarize', 'init']:
-        # For login/local commands, initialize empty connection
-        ctx.obj = ContextObject()
+    # Initialize ContextObject to be used for storing api object
+    ctx.obj = ContextObject()
+
+    if ctx.invoked_subcommand in ['compile', 'preview', 'summarize', 'init']:
+        # For local commands, initialize empty connection
         ctx.obj.api = Connection()
+    elif ctx.invoked_subcommand == 'login':
+        # Load analytics option from existing dotfile if present, else prompt
+        try:
+            api = Connection.from_file(config)
+            api.api_root = (
+                api_root or os.environ.get('BASE_URL', None) or api.api_root
+            )
+            ctx.obj.api = api
+        except (OSError, IOError):
+            ctx.obj.api = Connection()
+        # Echo a warning if other options are defined for login
+        if organization or email or token:
+            click.echo("Only the `--api-root` option is applicable for the "
+                       "`login` command. All other options are ignored.")
     else:
         try:
-            ctx.obj = ContextObject()
             api = Connection.from_file(config)
             api.api_root = (
                 api_root or os.environ.get('BASE_URL', None) or api.api_root
@@ -163,15 +177,18 @@ def cli(ctx, api_root, email, token, organization, config):
             )
             ctx.obj.api = api
         except (OSError, IOError):
-            click.echo("Welcome to TxPy! It seems like your `.transcriptic` config file is missing or out of date")
-            analytics = click.confirm("Send TxPy CLI usage information to improve the CLI user "
+            click.echo("Welcome to TxPy! It seems like your `.transcriptic` "
+                       "config file is missing or out of date")
+            analytics = click.confirm("Send TxPy CLI usage information to "
+                                      "improve the CLI user "
                                       "experience?", default=True)
             ctx.obj.api = Connection()  # Initialize empty connection
             ctx.invoke(login, api_root=api_root, analytics=analytics)
     if ctx.obj.api.analytics:
         try:
-            ctx.obj.api._post_analytics(event_action=ctx.invoked_subcommand, event_category="cli")
-        except:
+            ctx.obj.api._post_analytics(event_action=ctx.invoked_subcommand,
+                                        event_category="cli")
+        except requests.exceptions.RequestException:
             pass
 
 
@@ -1043,8 +1060,10 @@ def compile(protocol_name, args):
               help='Payment id to be used for run submission. '
                    'Use `transcriptic payments` command to list existing '
                    'payment methods.')
+@click.option('--test', help='Submit this run in test mode', is_flag=True)
 @click.pass_context
-def launch(ctx, protocol, project, save_input, local, accept_quote, params, pm=None):
+def launch(ctx, protocol, project, save_input, local, accept_quote, params,
+           pm=None, test=None):
     """Configure and launch a protocol either using the local manifest file or remotely.
     If no parameters are specified, uses the webapp to select the inputs."""
     # Validate payment method
@@ -1086,7 +1105,10 @@ def launch(ctx, protocol, project, save_input, local, accept_quote, params, pm=N
         if save_input:
             try:
                 with click.open_file(save_input, 'w') as f:
-                    f.write(json.dumps(quick_launch["inputs"], indent=2))
+                    f.write(
+                        json.dumps(dict(parameters=quick_launch["raw_inputs"]),
+                                   indent=2)
+                    )
             except Exception as e:
                 print_stderr("\nUnable to save inputs: %s" % str(e))
 
@@ -1109,10 +1131,13 @@ def launch(ctx, protocol, project, save_input, local, accept_quote, params, pm=N
                              "File is probably incorrectly formatted.")
                 return
 
-        req_id, launch_protocol = _get_launch_request(ctx, params, protocol_obj)
+        req_id, launch_protocol = _get_launch_request(
+            ctx, params, protocol_obj, test
+        )
 
         # Check for generation errors
         generation_errs = launch_protocol["generation_errors"]
+
         if len(generation_errs) > 0:
             for errors in generation_errs:
                 click.echo("\n\n" + str(errors["message"]))
@@ -1122,7 +1147,8 @@ def launch(ctx, protocol, project, save_input, local, accept_quote, params, pm=N
         # Confirm proceeding with purchase
         if not accept_quote:
             click.echo("\n\nCost Breakdown")
-            resp = ctx.obj.api.analyze_launch_request(req_id)
+            resp = ctx.obj.api.analyze_launch_request(req_id,
+                                                      test_mode=test)
             click.echo(price(resp))
             confirmed = click.confirm(
                 'Would you like to continue with launching the protocol',
@@ -1145,7 +1171,7 @@ def launch(ctx, protocol, project, save_input, local, accept_quote, params, pm=N
         try:
             req_json = ctx.obj.api.submit_launch_request(
                 req_id, protocol_id=protocol_obj["id"],
-                project_id=project, title=default_title, test_mode=None,
+                project_id=project, title=default_title, test_mode=test,
                 payment_method_id=pm
             )
             run_id = req_json['id']
@@ -1158,7 +1184,37 @@ def launch(ctx, protocol, project, save_input, local, accept_quote, params, pm=N
         if not params:
             run_protocol(manifest, protocol_obj, quick_launch["inputs"])
         else:
-            run_protocol(manifest, protocol_obj, json.load(params))
+            """
+            In the case of a local `launch`, we need to generate `inputs` from
+            `raw_inputs`, since the `run_protocol` function takes in JSON which 
+            is `inputs`-formatted.
+            `inputs` is basically an extended version of `raw_inputs`, where
+            we populate properties such as aliquot information for specified
+            containerIds.
+            In order to generate these `inputs`, we can create a new quick 
+            launch
+            """
+            try:
+                params = json.loads(params.read())
+                # This is the input format required by resolve_inputs
+                formatted_inputs = dict(inputs=params['parameters'])
+            except ValueError:
+                print_stderr("Unable to load parameters given. "
+                             "File is probably incorrectly formatted.")
+                return
+            quick_launch = ctx.obj.api.create_quick_launch(
+                data=json.dumps({"manifest": protocol_obj}),
+                project_id=project
+            )
+            quick_launch_obj = ctx.obj.api.resolve_quick_launch_inputs(
+                formatted_inputs,
+                project_id=project,
+                quick_launch_id=quick_launch['id']
+            )
+            inputs = quick_launch_obj['inputs']
+            run_protocol(
+                manifest, protocol_obj, inputs
+            )
 
 
 def _create_launch_request(params, bsl=1, test_mode=False):
@@ -1170,9 +1226,10 @@ def _create_launch_request(params, bsl=1, test_mode=False):
     return json.dumps(params_dict)
 
 
-def _get_launch_request(ctx, params, protocol):
+def _get_launch_request(ctx, params, protocol, test_mode):
     """Launches protocol from parameters"""
-    launch_request = _create_launch_request(params)
+    launch_request = _create_launch_request(params, test_mode=test_mode)
+
     launch_protocol = ctx.obj.api.launch_protocol(launch_request,
                                                   protocol_id=protocol["id"])
     launch_request_id = launch_protocol["id"]
@@ -1287,30 +1344,41 @@ def org_prompt(org_list):
 @click.pass_context
 def login(ctx, api_root=None, analytics=True):
     """Authenticate to your Transcriptic account."""
-    # For logging in, we should default to the transcriptic domain
     if api_root is None:
-        api_root = "https://secure.transcriptic.com"
+        # Always default to the pre-defined api-root if possible, else use
+        # the secure.transcriptic.com domain
+        try:
+            api_root = ctx.obj.api.api_root
+        except ValueError:
+            api_root = "https://secure.transcriptic.com"
+
     email = click.prompt('Email')
     password = click.prompt('Password', hide_input=True)
-    r = ctx.obj.api.post(
-        routes.login(api_root=api_root),
-        data=json.dumps({
-            'user': {
-                'email': email,
-                'password': password,
+    try:
+        r = ctx.obj.api.post(
+            routes.login(api_root=api_root),
+            data=json.dumps({
+                'user': {
+                    'email': email,
+                    'password': password,
+                },
+            }),
+            headers={
+                'Accept': 'application/json',
+                'Content-Type': 'application/json',
             },
-        }),
-        headers={
-            'Accept': 'application/json',
-            'Content-Type': 'application/json',
-        },
-        status_response={
-            '200': lambda resp: resp,
-            '401': lambda resp: resp,
-            'default': lambda resp: resp
-        },
-        custom_request=False
-    )
+            status_response={
+                '200': lambda resp: resp,
+                '401': lambda resp: resp,
+                'default': lambda resp: resp
+            },
+            custom_request=False
+        )
+    except requests.exceptions.RequestException:
+        click.echo("Error logging into specified host: {}. Please check your "
+                   "internet connection and host name".format(api_root))
+        sys.exit(1)
+
     if r.status_code != 200:
         click.echo("Error logging into Transcriptic: %s" % r.json()['error'])
         sys.exit(1)
@@ -1524,3 +1592,17 @@ def stdchannel_redirected(stdchannel, dest_filename):
             os.dup2(oldstdchannel, stdchannel.fileno())
         if dest_file is not None:
             dest_file.close()
+
+
+# Placing this here since this is exclusively used by the CLI currently
+class ProtocolPreview(object):
+    """
+    An object for previewing protocols
+    """
+    def __init__(self, protocol, connection):
+        self.protocol = protocol
+        self.preview_url = connection.preview_protocol(protocol)
+
+    def _repr_html_(self):
+        return """<iframe src="%s" frameborder="0" allowtransparency="true" \
+        style="height:500px" seamless></iframe>""" % self.preview_url
